// AtCommands.h
// Read AT commands on an Arduino
// Assumes Serial has already been started
// 2009 Lucas Eckels
// http://lucaseckels.com

#include <ctype.h>

#define MAX_LINE_LENGTH 16
char lineBuffer[MAX_LINE_LENGTH];
byte lineBufferPos = 0;

int atCommandsGetDigits(int count = 2) {
  while (Serial.available() < count) {
    // Wait
  }
  int val = 0;
  for (int i = 0; i < count; ++i) {
    val *= 10;
    val += Serial.read() - '0';
  }
  return val;
}

void atCommandsGetUpperString(char *buffer, int count) {
  while (Serial.available() < count) {
    // Wait
  }
  for (int i = 0; i < count; ++i) {
    buffer[i] = toupper(Serial.read());
  }
}

void atCommandError(char *reason) {
  Serial.print("ERROR: ");
  Serial.print(reason);
  Serial.print("\r\n");
  lineBufferPos = 0;
}

void atCommandOk() {
  Serial.print("OK\r\n");
  lineBufferPos = 0;
}

#define AT_COMMAND_MAP_BEGIN(name)            \
static void atCommandParse##name();           \
static void atCommandHandler##name() {        \
  while (Serial.available()) {                \
    byte b = Serial.read();                   \
    lineBuffer[lineBufferPos] = toupper(b);   \
    if (++lineBufferPos >= MAX_LINE_LENGTH) { \
      atCommandError("line overflow");        \
      continue;                               \
    }                                         \
    if (b == '\n' || b == '\r') {             \
      --lineBufferPos;                        \
      lineBuffer[lineBufferPos] = '\0';       \
      atCommandParse##name();                 \
      return;                                 \
    }                                         \
  }                                           \
}                                             \
                                              \
void atCommandParse##name() {                 \
  if (memcmp("AT", lineBuffer, 2)) {          \
    atCommandError("didn't start with at");   \
    return;                                   \
  }
  
#define AT_COMMAND_MAP_ENTRY(cmd, function)   \
  if (memcmp(cmd, lineBuffer+2, 2) == 0) {    \
    function(cmd, lineBuffer+4, lineBufferPos-4);  \
    lineBufferPos = 0;                        \
    return;                                   \
  } \
  
#define AT_COMMAND_MAP_END(name)              \
  atCommandError("unknown command");          \
}
